#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# This script uses the merged CRU and University of Delaware
# datasets to split the panel dataset into a two-level list, with
# one splitting crops, and second level splitting climate datasets.
# For example, the list, called 'panels', contains all the
# observations of maize with CRU weather data in their own
# compartment, e.g., panels[['maize']][['CRU']].
#
# This script also changes the eleven crops into six by scrapping
# the second maize and sorghum seasons, combining the rice seasons
# for the countries with significant quantities of rice grown in
# its 2nd season, and using either winter or spring for wheat 
# and barley, depending on which season is dominant in a given
# country.
#
# Finally, each list member has any suspicious observations flagged,
# In particular, we check for strongly nonlinear trends in yields
# over time, growing areas too small for analysis, and identical
# yields being reported for three or more years running. If a
# country is flagged, all its observations in that list member
# are thrown out.
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#



rm(list=ls())
setwd('Z:\\Lobell_Lab')
qgdir <- 'Z:\\Lobell_Lab\\world_panels\\quantile-grouping\\'
#load('world_panels\\panels.RData')
orig.panel <- read.csv(paste(qgdir,'panels_half_deg_area\\udel.cru.panel.6crop.nayield.1961.2008.csv',
                        sep=''), header=T)

### Combine and scrap seasons. This has to be done here rather
### than in the panels.RData set because the code that runs the
### model references 'orig.panel' as well as 'panels'.

### remove second season maize
maize2Indices <- which(orig.panel$Crop == 'maize2')
orig.panel <- orig.panel[-maize2Indices,]

### remove second season sorghum
sorghum2Indices <- which(orig.panel$Crop == 'sorghum2')
orig.panel <- orig.panel[-sorghum2Indices,]

### combine wheat seasons
wheat2Indices <- which(orig.panel$Crop=='wheat2')
springWheatCountries <- c('Canada', 'Armenia', 'Azerbaijan', 'Bhutan',
                          'Byelarus', 'Estonia', 'Georgia', 'Kazakhstan',
                          'Kyrgyzstan', 'Latvia', 'Lithuania', 'Moldova',
                          'Mongolia', 'Nepal', 'North Korea', 'Russia',
                          'Tajikistan', 'Turkmenistan', 'Ukraine', 'Uzbekistan',
                          'Australia','New Zealand', 'South Africa', 'Zimbabwe')

for (i in wheat2Indices) {
  if (orig.panel[i,'Country'] %in% springWheatCountries) {
      orig.panel[i,] <-
          orig.panel[which(orig.panel[,'Country'] == orig.panel[i,'Country'] &
                           orig.panel[,'Year'] == orig.panel[i,'Year'] &
                           orig.panel[,'Crop'] == 'wheat'),]
          orig.panel[i,'Crop'] <- 'wheat2'
  }
}
wheatIndices <- which(orig.panel$Crop=='wheat')
orig.panel <- orig.panel[-wheatIndices,]     # remove first season wheat
wheat2Indices <- which(orig.panel$Crop=='wheat2')
orig.panel$Crop[wheat2Indices] <- 'wheat'

### combine barley seasons---same as wheat
barley2Indices <- which(orig.panel$Crop=='barley2')
springBarleyCountries <- c('Canada', 'Armenia', 'Azerbaijan', 'Bhutan',
                          'Byelarus', 'Estonia', 'Georgia', 'Kazakhstan',
                          'Kyrgyzstan', 'Latvia', 'Lithuania', 'Moldova',
                          'Mongolia', 'Nepal', 'North Korea', 'Russia',
                          'Tajikistan', 'Turkmenistan', 'Ukraine', 'Uzbekistan',
                          'Australia','New Zealand', 'South Africa', 'Zimbabwe')

for (i in barley2Indices) {
  if (orig.panel[i,'Country'] %in% springBarleyCountries) {
      orig.panel[i,] <-
          orig.panel[which(orig.panel[,'Country'] == orig.panel[i,'Country'] &
                           orig.panel[,'Year'] == orig.panel[i,'Year'] &
                           orig.panel[,'Crop'] == 'barley'),]
          orig.panel[i,'Crop'] <- 'barley2'
  }
}
barleyIndices <- which(orig.panel$Crop=='barley')
orig.panel <- orig.panel[-barleyIndices,]     # remove first season barley
barley2Indices <- which(orig.panel$Crop=='barley2')
orig.panel$Crop[barley2Indices] <- 'barley'

###  combine rice seasons
rice2countries <- c('Bangladesh', 'China', 'India', 'Indonesia', 'Malaysia',
               'Myanmar', 'Philippines', 'Taiwan', 'Thailand')
mainWeight <- c(0.8, 0.66, 0.8, 0.8, 0.5, 0.8, 0.8, 0.8, 0.8)
secondWeight <- 1 - mainWeight
climVars <- c('precCRU', 'tMinCRU', 'tMaxCRU', 'tAvgCRU', 'tAvgUDel', 'precUDel')
for (cntry in 1:length(rice2countries)) {
  index <- which(orig.panel$Crop == 'rice' &
                 orig.panel$Country == rice2countries[cntry])
  for (i in index) {
    secondIndex <- which(orig.panel$Crop=='rice2' &
                         orig.panel$Country==rice2countries[cntry]  &
                         orig.panel$Year==orig.panel[i,'Year'])
    orig.panel[i,climVars] <- mainWeight[cntry]*orig.panel[i,climVars] +
                              secondWeight[cntry]*orig.panel[secondIndex,climVars]
  }
}
rice2Indices <- which(orig.panel$Crop=='rice2')
orig.panel <- orig.panel[-rice2Indices,]     # remove second season rice

### remove empty levels from 'Crop' column
orig.panel$Crop <- factor(orig.panel$Crop)

save(orig.panel, file = paste(qgdir,'panels_half_deg_area\\fullCompositeSeasonPanelsNAYield_6crop.RData', sep=''))
write.csv(orig.panel, file=paste(qgdir,'panels_half_deg_area\\fullCompositeSeasonPanelsNAYield_6crop.csv', sep=''),
          row.names=F)



######## flag and process into lists #########

load(paste(qgdir,'panels_half_deg_area\\fullCompositeSeasonPanelsNAYield_6crop.RData', sep=''))
athresh <- 1e4
panels <- list()
crops <- c('maize','rice','wheat','soy','barley','sorghum')
reg.names <- paste('Q',1:4, sep='')

# function to extract x~time residuals
get.res <- function(x) lm(x~c(1:length(x)))$resid

for (cr in crops) {
  panels[[cr]] <- list()
  f.crop <- orig.panel[which(orig.panel$Crop == cr),]
  yield <- f.crop$Yield
  logY <- f.crop$logYield
  t <- f.crop$Year
  t2 <- f.crop$Year^2
  country <- f.crop$Country
  region <- f.crop$Region
  fao_code <- f.crop$FAO_code
  area <- f.crop$Area
  production <- f.crop$Production

  # manually split by data source
  f.udel <- data.frame(country, fao_code, region, t, t2, logY,
                      yield, area, production, tmax = f.crop$tMaxCRU,
                      tmax2 = f.crop$tMaxCRU^2,
                      prec = f.crop$precUDel, prec2 = f.crop$precUDel^2,
                      tavg = f.crop$tAvgUDel, tavg2 = f.crop$tAvgUDel^2)

  f.cru <- data.frame(country, fao_code, region, t, t2, logY,
                      yield, area, production,
                      prec = f.crop$precCRU, prec2 = f.crop$precCRU^2,
                      tavg = f.crop$tAvgCRU, tavg2 = f.crop$tAvgCRU^2,
                      tmin = f.crop$tMinCRU, tmin2 = f.crop$tMinCRU^2,
                      tmax = f.crop$tMaxCRU, tmax2 = f.crop$tMaxCRU^2)

  # Get rid of NA's for UDel set
  f.udel <- f.udel[which(is.finite(f.udel$logY*f.udel$tavg)),]
  # Same for CRU
  f.cru <- f.cru[which(is.finite(f.cru$logY*f.cru$tavg)),]

  # flag countries with suspect yields
  # uses CRU because CRU years are subset of UDel years
  f.cru$flag <- rep(0,nrow(f.cru))
  f.udel$flag <- rep(0,nrow(f.udel))
  for (cntry in levels(f.cru$country)) {
    indexCRU <- which(f.cru$country==cntry)
    indexUDel <- which(f.udel$country==cntry)
    if (sum(diff(f.cru$logY[indexCRU])==0) > 2) {
      f.cru$flag[indexCRU] <- 1
      f.udel$flag[indexUDel] <- 1
    }
    if (length(indexCRU)>12) {
      acfstat <- max(abs(acf(get.res((f.cru$logY[indexCRU])),plot=F)$acf[9:12]))
      if (acfstat>.35 | is.nan(acfstat)) {
        f.cru$flag[indexCRU] <- 1
        f.udel$flag[indexUDel] <- 1
      }
    }
    meanarea <- mean(f.cru$area[indexCRU],na.rm=T)
    if (!is.na(meanarea)) {
      if (meanarea<athresh) {
        f.cru$flag[indexCRU] <- 1
        f.udel$flag[indexUDel] <- 1
      }
    }
    if (cntry == 'Rest Of World') {
      f.cru$flag[indexCRU] <- 1
      f.udel$flag[indexUDel] <- 1
    }
  }

  panels[[cr]][['UDel']] <- f.udel
  panels[[cr]][['CRU']] <- f.cru
}

for (cr in crops) {
  tmpSet <- panels[[cr]][['UDel']]
  tmpSet <- tmpSet[-which(tmpSet$t > 2002),]
  panels[[cr]][['UDel02']] <- tmpSet
}

save(panels, file=paste(qgdir,'panels_half_deg_area\\compositeSeasonPanelsNAYield_6crop.RData', sep=''))