#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# This script extracts the CRU climate data for grid cells in which
# the crops of interest are grown.
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#

library(raster)
library(RNetCDF)
setwd('Z:\\Lobell_Lab')

### Define crop names and associated files
climVars = c('prec', 'tmin', 'tmax')    # variables to be extracted
crops = c('maize','maize2','rice','rice2','wheat','wheat2','soy','barley','barley2','sorghum','sorghum2')
areanames = c('maize','maize','rice','rice','wheat','wheat','soy','barley','barley','sorghum','sorghum')
calnames = c('Maize','Maize.2','Rice','Rice.2','Wheat','Wheat.Winter','Soybeans',
             'Barley','Barley.Winter','Sorghum','Sorghum.2')
unique.crop.cals = c(1, 3, 5, 7, 8, 10)

### Read in 5 min crop area files
areadir = 'SAGE_crop_calendars\\Ramankutty_maps\\'

### Read in 5 min sow date files
caldir = 'SAGE_crop_calendars\\'

### Takes a planting and harvest date (or a vector of each), and returns a
### vector of length 24 (or an array, if inputs are vectors), one for each
### month in two years, with the fraction of each month during which the crop
### was growing, i.e., the weighting of months for extracting weather data.
get.mo = function(d1,d2) {  #input can be a vector, output is a matrix of length(d1)*24
  nday = length(d1)
  out = array(0,dim=c(nday,24))
  for (d in 1:nday) {
    m1 = (max(which((d1[d]-sday) >= 0))) #start month with any days
    m2 = max(which((d2[d]-sday) >= 0))   #last month with any days
    if (m2>m1) {
      out[d,] = c(rep(0,12+m1-1),(sday[m1+1]-d1[d])/(sday[m1+1]-sday[m1]),rep(1,m2-m1-1),
                 (d2[d]-sday[m2])/(sday[m2+1]-sday[m2]),rep(0,12-m2))
    } else {
      out[d,] = c(rep(0,m1-1),(sday[m1+1]-d1[d])/(sday[m1+1]-sday[m1]),rep(1,m2+12-m1-1),
                 (d2[d]-sday[m2])/(sday[m2+1]-sday[m2]),rep(0,12-m2))
    }
  }
  out
}

### Function to read in which CRU cells are not NA, i.e., which are land
getLandCells = function(pre) {
  landCells = array(dim=c(67420,2))
  lcIndex = 1
  for (j in 1:360) {
    for (i in 1:720) {
      if (!is.na(pre[i,j])) {
        landCells[lcIndex,] = c((i-360)/2-.25,(j-180)/2-.25)
        lcIndex = lcIndex + 1
      }
    }
  }
  landCells
}

### Format netcdf data the same way UDel data is formatted
netcdfToUDel = function(ncfile, ncvar, yr, latLongArr) {
  yrArr = array(dim=c(nrow(latLongArr),14))
  yrArr[,1:2] = latLongArr  # latitude and longitude of cell
  for (month in 1:12) {
    monthLayer = var.get.nc(ncfile, ncvar, start=c(1,1,(12*(yr-1) + month)), count = c(720,360,1))
    yrArr[,(month+2)] = sapply(1:nrow(latLongArr), function(x) {
        .1*monthLayer[((latLongArr[x,1]+.25)*2 +360),((latLongArr[x,2]+.25)*2+180)]})
    # have to multiply by .1 because data is 10x; latitude and longitude
    # are converted into indices 1:360 & 1:720, respectively
  }
  yrArr
}



### Aggregate area maps up to climate data scale
### This only needs to be done once and takes awhile, so skips if files already exist
for (cr in unique.crop.cals) {
  if (!any(dir('world_panels\\aggregated_5min_maps')==paste('areamap',crops[cr],
                                             'RData', sep='.'))) {
    print(paste('Aggregating', crops[cr], 'area maps'))
    temp = open.nc(paste(areadir,areanames[cr],'_5min.nc',sep=''))
    a = var.get.nc(temp,'cropdata')
    a = t(a[,,1])   # take first level
    ra = raster(ncol=4320, nrow=2160)
    ra[] = a   # need to make space for 'ra'
    ra = ra*area(ra)
    ra = aggregate(ra, fact=6, fun=mean, na.rm=T) # 5-minute*6 = 0.5 deg
    ra = calc(ra, fun=function(x) {x[is.nan(x)] = NA; return(x)})
    save(ra, file=paste('world_panels\\aggregated_5min_maps\\areamap',crops[cr],'RData', sep='.'))
    rm(ra)
  } else {
    next
  }
}

### Read in CRU data, match to crop area maps and crop calendars
for (climVar in climVars) {
  temp = open.nc('world_panels\\country.fid.0.5x0.5.nc')
  fid = var.get.nc(temp,'fid')
  r1 = raster(ncol=720,nrow=360)
  r1[] = t(fid[,ncol(fid):1])
  close.nc(temp)
  fids = unique(r1)
  fids = fids[fids!=1]  #THROW OUT OCEAN CELLS
  nfid = length(fids)

  # Read in one year of UDel data to determine which rows are which FID
  years = 1952:2002
  nyr = length(years)
  filename = paste('CRU\\mon.lan.', climVar, '_1951-1980.nc', sep='')
  file = open.nc(filename)
  pre = var.get.nc(file, climVar, start=c(1,1,1), count=c(720,360,1)) #reads in all UDEL data for that year

  landCells = getLandCells(pre)

  pre = netcdfToUDel(file, climVar, 1, landCells)   # 'pre' is now in UDel format

  cells <- cellFromXY(r1, pre[,1:2]) #finds which cells correspond to rows of data
  fid.rows=list()
  for (i in 1:nfid) fid.rows[[i]]=which(r1[cells]==fids[i])


  d.in.mo=c(31,28,31,30,31,30,31,31,30,31,30,31)
  sday=cumsum(c(1,d.in.mo[1:12]))

  # Save indices, area weights, and sow/harvest date for cells in each country.
  # Instead of saving dates, saves and array that has ncells by month weights for a 24-mo period.
  # First removes any cells that have NA for growing season;
  # also rounds up fractional days, to avoid problems with planting on day 0.5

  ncrops = length(crops)
  save.crop.map=list()
  scale = sapply(1:360, function(x) acos(sin(2*pi/360*((x-180)/2-.25))^2 + cos(2*pi/360*((x-180)/2-.25))^2*cos(2*pi/360*1))*360/2/pi)
  scaleArr = array(dim=c(360,720))
  for (i in 1:720) scaleArr[,i] = scale
  for (cr in 1:ncrops) {
    load(paste('world_panels\\aggregated_5min_maps\\areamap',crops[cr],'RData', sep='.'))
    temp = open.nc(paste(caldir,calnames[cr],'.crop.calendar.fill.nc',sep=''))
    harv = var.get.nc(temp,8)
    rhv = raster(ncol=720,nrow=360)
    rhv[] = ceiling(t(harv))
    rpl = raster(ncol=720,nrow=360)
    if (substr(crops[cr], 1, 5)=='wheat') {
      rpl = (rhv - 120) %% 365
    } else {
      plant = var.get.nc(temp,4)
      rpl[] = ceiling(t(plant))
    }
    save.crop.map[[cr]] = lapply(fid.rows, function(x) {
      not.na = which(is.finite(rpl[cells[x]]*rhv[cells[x]]))
      x = x[not.na]
      if (length(x) >1)
        list(x,ra[cells[x]],get.mo(rpl[cells[x]],rhv[cells[x]]))
      else
        list(NA,NA,NA)
    })
  }

  hold = array(dim=c(nfid,ncrops,nyr))
  pb <- winProgressBar(min=0,max=nyr)
  for (yr.i in 2:(nyr+1)) { # start out with 2nd year in netcdf file. Starts at
    # yr.i==2 because this way the yr.i variable still works to index netcdf
    if (yr.i == 31) {   # second file begins at yr 31, i.e. 1981
      filename = paste('CRU\\mon.lan.',climVar,'_1981-2002.nc',sep='')
      file = open.nc(filename)
      # need to update landCells for new dataset
      landCells = getLandCells(var.get.nc(file, climVar, start=c(1,1,1),
                                          count=c(720,360,1)))
    }
    # use ternary operator here because yr 31 is actually yr 1 in new data set
    a = netcdfToUDel(file, climVar, ifelse(yr.i<31,yr.i,yr.i-30), landCells)
    comb = cbind(pre[,-c(1:2)],a[,-c(1:2)])
    for (cr in 1:ncrops) hold[,cr,(yr.i-1)] = sapply(save.crop.map[[cr]], function(x) {
    if (length(x[[1]])>1)(sum(apply(comb[x[[1]],]*x[[3]],1,sum, na.rm=T)/apply(x[[3]],1,sum, na.rm=T) * (x[[2]]), na.rm=T)/sum(x[[2]], na.rm=T)) else NA })
    pre = a
    setWinProgressBar(pb,yr.i)
  }
  close(pb)   # begone, irritating progress bar

  dimnames(hold) = list(fids,crops,years)
  save(hold,file=paste('world_panels\\quantile-grouping\\panels_half_deg_area\\cru.6crop',
                       climVar,years[1],years[nyr],'RData',sep='.'))
}



### Combine 'hold' files of each variable into one dataset for all CRU variables
df = data.frame(array(dim=c(length(hold[,1,1])*nyr*ncrops,5+length(climVars))))
colnames(df) = c('Country', 'FID', 'FAO_code', 'Year', 'Crop', climVars)
ccodes = read.csv('world_panels\\cntry.codes.all.csv', header=T)
start.yr = 1952
cntryIDs = as.numeric(names(hold[,1,1]))
cntryNames = ccodes[match(cntryIDs, ccodes[,1]),'gmi_cntry_name']
faoCodes = ccodes[match(cntryIDs, ccodes[,1]),'fao_code']
df$Country = rep(cntryNames,rep(nyr*ncrops,length(cntryNames)))
df$FID = rep(cntryIDs,rep(nyr*ncrops,length(cntryNames)))
df$FAO_code = rep(faoCodes,rep(nyr*ncrops,length(cntryNames)))
df$Year = rep(rep(years,rep(ncrops,nyr)),length(cntryNames))
df$Crop = rep(crops, length(cntryNames)*nyr)
for (climVar in climVars) {
  rm(hold)
  load(paste('world_panels\\quantile-grouping\\panels_half_deg_area\\cru.6crop.',
       climVar,'.1952.2002.RData', sep=''))
  df[,climVar] = unlist(lapply(1:length(cntryNames), function(x) as.vector(hold[x,,])))
}
write.csv(df, 'world_panels\\quantile-grouping\\panels_half_deg_area\\cru.6crop.panel.1952.2002.csv',
          row.names=F)