#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# This script extracts the University of Delaware climate data for grid
# cells in which the crops of interest are grown.
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#

library(raster)
library(RNetCDF)
setwd('Z:\\Lobell_Lab')

### Define crop names and associated files
climVars = c('air_temp', 'precip')    # variables to be extracted
crops = c('maize','maize2','rice','rice2','wheat','wheat2','soy','barley','barley2','sorghum','sorghum2')
ncrops = length(crops)
areanames = c('maize','maize','rice','rice','wheat','wheat','soybean','barley','barley','sorghum','sorghum')
calnames = c('Maize','Maize.2','Rice','Rice.2','Wheat','Wheat.Winter','Soybeans',
             'Barley','Barley.Winter','Sorghum','Sorghum.2')
unique.crop.cals = c(1, 3, 5, 7, 8, 10)

### Read in 5 min crop area files
areadir = 'SAGE_crop_calendars\\Ramankutty_maps\\'

### Read in 5 min sow date files
caldir = 'SAGE_crop_calendars\\'

### Takes a planting and harvest date (or a vector of each), and returns a
### vector of length 24 (or an array, if inputs are vectors), one for each
### month in two years, with the fraction of each month during which the crop
### was growing, i.e. the weighting of months for extracting weather data.
get.mo = function(d1,d2) {  #input can be a vector, output is a matrix of length(d1)*24
  nday = length(d1)
  out = array(0,dim=c(nday,24))
  for (d in 1:nday) {
    m1 = (max(which((d1[d]-sday) >= 0))) #start month with any days
    m2 = max(which((d2[d]-sday) >= 0))   #last month with any days
    if (m2>m1) {
      out[d,] = c(rep(0,12+m1-1),(sday[m1+1]-d1[d])/(sday[m1+1]-sday[m1]),rep(1,m2-m1-1),
                 (d2[d]-sday[m2])/(sday[m2+1]-sday[m2]),rep(0,12-m2))
    } else {
      out[d,] = c(rep(0,m1-1),(sday[m1+1]-d1[d])/(sday[m1+1]-sday[m1]),rep(1,m2+12-m1-1),
                 (d2[d]-sday[m2])/(sday[m2+1]-sday[m2]),rep(0,12-m2))
    }
  }
  out
}



### Aggregate area maps up to climate data scale
### This only needs to be done once and takes awhile, so skips if files already exist
for (cr in 1:ncrops) {
  if (!any(dir('world_panels\\aggregated_5min_maps')==paste('areamap',crops[cr],
                                             'RData', sep='.'))) {
    print(paste('Aggregating', crops[cr], 'area maps'))
    temp=open.nc(paste(areadir,areanames[cr],'_5min.nc',sep=''))
    a = var.get.nc(temp,'cropdata')
    a = t(a[,,1])   # take first level
    ra = raster(ncol=4320, nrow=2160)
    ra[] = a   # need to make space for 'ra'
    ra = ra*area(ra)
    ra = aggregate(ra, fact=6, fun=mean, na.rm=T) # 5-minute*6 = 0.5 deg
    ra = calc(ra, fun=function(x) {x[is.nan(x)] = NA; return(x)})
    save(ra, file=paste('world_panels\\aggregated_5min_maps\\areamap',crops[cr],'RData', sep='.'))
    rm(ra)
  } else {
    next
  }
}

### Read in UDel data, match to crop area maps and crop calendars
for (climVar in climVars) {
  temp = open.nc('world_panels\\country.fid.0.5x0.5.nc')
  fid = var.get.nc(temp,'fid')
  r1 = raster(ncol=720,nrow=360)
  r1[] = t(fid[,ncol(fid):1])
  close.nc(temp)
  fids = unique(r1)
  fids = fids[fids!=1]  # throw out ocean cells
  nfid = length(fids)

  # Read in one year of UDel data to determine which rows are which FID
  years = 1959:2008
  nyr = length(years)
  filename = paste('UDel\\',climVar,'2009\\',climVar,'.',years[1]-1,sep='')
  pre = read.table(filename) # reads in all UDEL data for that year
  cells <- cellFromXY(r1, pre[,1:2]) # finds which cells correspond to rows of udel data
  fid.rows = list()
  for (i in 1:nfid) fid.rows[[i]] = which(r1[cells]==fids[i])

  d.in.mo = c(31,28,31,30,31,30,31,31,30,31,30,31)
  sday = cumsum(c(1,d.in.mo[1:12]))

  # Save indices, area weights, and sow/harvest date for cells in each country.
  # Instead of saving dates, saves and array that has ncells by month weights for a 24-mo period.
  # First removes any cells that have NA for growing season;
  # also rounds up fractional days, to avoid problems with planting on day 0.5

  save.crop.map=list()
  for (cr in 1:ncrops) {
    load(paste('world_panels\\aggregated_5min_maps\\areamap',crops[cr],'RData', sep='.'))
    temp = open.nc(paste(caldir,calnames[cr],'.crop.calendar.fill.nc',sep=''))
    harv = var.get.nc(temp,8)
    rhv = raster(ncol=720,nrow=360)
    rhv[] = ceiling(t(harv))
    rpl = raster(ncol=720,nrow=360)
    # for wheat only, use just 120 days prior to harvest, ignoring plant date
    if (substr(crops[cr], 1, 5)=='wheat') {
      rpl = (rhv - 120) %% 365
    } else {
      plant = var.get.nc(temp,4)
      rpl[] = ceiling(t(plant))
    }
    save.crop.map[[cr]]=lapply(fid.rows, function(x) {
      not.na = which(is.finite(rpl[cells[x]]*rhv[cells[x]]))
      x = x[not.na]
      if (length(x) >1)
        list(x,ra[cells[x]],get.mo(rpl[cells[x]],rhv[cells[x]]))
      else
        list(NA,NA,NA)
    })
  }

  # Now for each year, extract cell values for a given country,
  # take months of interest depending on sow/harvest date
  # and then spatially average based on crop area map
  hold=array(dim=c(nfid,ncrops,nyr))
  pb <- winProgressBar(min=0,max=nyr)

  # Have already read in year prior to start, to account for growing seasons that start in prior year.
  # Takes relevant rows from UDel data and multiplies by array of same size
  # that has fraction planted in appropriate months, sums across all months,
  # then multiplies by vector that has fraction of area in each cell and sums up.
  for (yr.i in 1:nyr) {
    filename=paste('UDel\\',climVar,'2009\\',climVar,'.',years[yr.i],sep='')
    a=read.table(filename) #reads in all UDel data for that year
    comb=cbind(pre[,-c(1:2)],a[,-c(1:2)])
    for (cr in 1:ncrops) hold[,cr,yr.i] = sapply(save.crop.map[[cr]], function(x) {
      if (length(x[[1]])>1)(sum(apply(comb[x[[1]],]*x[[3]],1,sum, na.rm=T)/apply(x[[3]],1,sum, na.rm=T) * (x[[2]]), na.rm=T)/sum(x[[2]], na.rm=T)) else NA })
    pre=a
    setWinProgressBar(pb,yr.i)
  }
  dimnames(hold)=list(fids,crops,years)
  save(hold,file=paste('world_panels\\quantile-grouping\\panels_half_deg_area\\udel.6crop',
                       climVar,years[1],years[nyr],'RData',sep='.'))
}

### Format data, assign countries to grid cells, write to file
df = data.frame(array(dim=c(length(hold[,1,1])*nyr*ncrops,5+length(climVars))))
colnames(df) = c('Country', 'FID', 'FAO_code', 'Year', 'Crop', climVars)
ccodes = read.csv('world_panels\\cntry.codes.all.csv', header=T)
start.yr = 1952
cntryIDs = as.numeric(names(hold[,1,1]))
cntryNames = ccodes[match(cntryIDs, ccodes[,1]),'gmi_cntry_name']
faoCodes = ccodes[match(cntryIDs, ccodes[,1]),'fao_code']
df$Country = rep(cntryNames,rep(nyr*ncrops,length(cntryNames)))
df$FID = rep(cntryIDs,rep(nyr*ncrops,length(cntryNames)))
df$FAO_code = rep(faoCodes,rep(nyr*ncrops,length(cntryNames)))
df$Year = rep(rep(years,rep(ncrops,nyr)),length(cntryNames))
df$Crop = rep(crops, length(cntryNames)*nyr)
for (climVar in climVars) {
  rm(hold)
  load(paste('world_panels\\quantile-grouping\\panels_half_deg_area\\udel.6crop.',
             climVar,'.1959.2008.RData', sep=''))
  df[,climVar] = unlist(lapply(1:length(cntryNames), function(x) as.vector(hold[x,,])))
}
write.csv(df, file='world_panels\\quantile-grouping\\panels_half_deg_area\\udel.6crop.panel.1959.2008.csv',
          row.names=F)
