#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# This script estimates both linear an quadratic models predicting
# yields on a host of time and climate variables. First the necessary
# functions are defined; then at the end of the script they are run. The
# model is a fixed effects model, bootstrapped over all observations
# of a given crop within a region (using just one historical climate
# dataset at a time).
#
# In this particular script, the estimated coefficients are applied to
# datasets with tavg changed by -3 to 3 degrees C, so that the
# output includes bootstrapped mean estimates (and CI's) of yield
# changes for a set of (globally uniform) temperature increases.
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#

rm(list=ls())
setwd('Z:\\Lobell_Lab')
qgdir <- 'Z:\\Lobell_Lab\\world_panels\\quantile-grouping\\'
load(paste(qgdir,'panels_half_deg_area\\compositeSeasonPanelsNAYield_6crop.RData', sep=''))
load(paste(qgdir,'panels_half_deg_area\\fullCompositeSeasonPanelsNAYield_6crop.RData',sep=''))
crops <- names(panels)  # crop names
reg.names <- paste('Q',1:4,sep='')
delt <- -3:3
di <- duplicated(orig.panel$Country)
nreg <- length(reg.names)

### Compute average of specified variable by country for a specified
### crop and year range (used to aggregate nations into regions)
get.avg <- function(crop.i,colmn,yrs=c(1998,2002),plot.flag=0) {
  temp <- orig.panel[which(orig.panel$Crop==crops[crop.i]),]
  out <- data.frame(country=orig.panel$FAO_code[!di],value=NA)
  for (i in 1:nrow(out)){
    temp2 <- temp[which(temp$FAO_code==out[i,1]),]
    yeari <- which(temp2$Year>=yrs[1]&temp2$Year<=yrs[2])
    out[i,2] <- mean(temp2[yeari,colmn],na.rm=T)
  }
  if (is.character(colmn))
    names(out)[2] <- colmn
  else
    names(out)[2] <- names(orig.panel)[colmn]
  out
}

### Predict yields for temperature changes specified in
### parameter 'delt'
pred.fix <- function(fit,arr,delt) {
  country <- unique(arr$country)[order(unique(arr$country))]
  out <- data.frame(country)

  # need matrix that will interact properly with 'fit'
  if (any(names(fit) == 'tavg2'))
    quadratic <- T
  else
    quadratic <- F

  data <- arr

  if (quadratic)
    arr <- model.matrix(lm(logY~tavg+tavg2+prec+prec2+country+country:t+country:t2-1, data=arr))
  else
    arr <- model.matrix(lm(logY~tavg+prec+country+country:t+country:t2-1, data=arr))

  newArr <- data.frame(arr)
  newyNoChange <- as.numeric(fit %*% t(newArr))

  for (dt in 1:length(delt)) {
    newArr <- data.frame(arr)
    newArr$tavg <- newArr$tavg + delt[dt]
    if (quadratic)
      newArr$tavg2 <- (newArr$tavg)^2
    newy <- as.numeric(fit %*% t(newArr))  # do not include 'country' var
    # compute average of raw yields, not of log yields
    temp <- sapply(country, function(x) {
      cntryIdx <- which(data.frame(data)$country==x)
      pctChY <- (exp(newy[cntryIdx]) - exp(newyNoChange[cntryIdx])) /
                                        exp(newyNoChange[cntryIdx])
      return(mean(pctChY,na.rm=T))
    })
    out <- cbind(out,temp)
  }

  names(out)[-1] <- paste('plus',delt,sep='')
  out <- out[which(!is.nan(out[,2])),]
  out
}


### Bootstrap over all observations
BootstrapModels <- function(dataset, n, delt) {
  startTime <- Sys.time()
  if (!any(delt == 0)) {
    delt <- c(0, delt)
    warning('\'delt\' did not include a leading \'0\', so one was added.')
  }
  model <- list()

  crops <- names(dataset)
  for (cr in crops) {
    model[[cr]] <- list()
    for (set in names(dataset[[1]])) {
      model[[cr]][[set]] <- list()
      for (reg in 1:nreg) {
        print(paste(cr, set, reg))
        model[[cr]][[set]][[reg]] <- list()
        f.temp <- dataset[[cr]][[set]][which(dataset[[cr]][[set]]$region==reg),]
        use <- which(f.temp$flag==0 & f.temp$country != 'Belgium'
                    & f.temp$country != 'Luxembourg')
        f.temp <- f.temp[use,]
        countryNames <- unique(f.temp$country)
        countryNames <- countryNames[order(countryNames)]

        for (ordr in 1:2) {
          if (ordr == 1)    # not necessary, but makes code cleaner later
            quadratic <- F
          else
            quadratic <- T

          # define model formula
          if (quadratic)
            formula <- logY~tavg+tavg2+prec+prec2+country+country:t+country:t2-1
          else
            formula <- logY~tavg+prec+country+country:t+country:t2-1

          if (length(use) == 0 | length(countryNames) <= 1) { # skip regions with 0 or 1 countries
            model[[cr]][[set]][[reg]][[ordr]] <- list(NA,NA,NA)
            next
          }


          nVars <- 2 + quadratic*2 + 3*length(unique(f.temp$country))
          results <- array(dim=c(n,nVars))
          results[,] <- 0
          predictions <- array(dim=c(length(countryNames), length(delt), n))
          predictions[,,] <- 0
          df <- nrow(f.temp) - nVars

          for (i in 1:n) {
            while(TRUE) {
              smpl <- sample(1:nrow(f.temp), nrow(f.temp), replace=T)
              if (length(unique(f.temp[smpl,'country'])) == 1)
                next
              # which countries left out of sample
              leftOut <- setdiff(countryNames, unique(f.temp[smpl,'country']))
              leaveOutIndex <- which(countryNames %in% leftOut) # store their indices
              if (length(leaveOutIndex) > 0)  {# handle case where none left out
                leaveOutIndex <- c(2+2*quadratic+leaveOutIndex,
                                  2+2*quadratic+length(countryNames)+leaveOutIndex,
                                  2+2*quadratic+2*length(countryNames)+leaveOutIndex)
              }
              data <- f.temp[smpl,]
              predictions[which(countryNames %in% leftOut),,i] <- NA
              fit <- lm(formula=formula, data=data)
              tmpPredict <- pred.fix(fit$coef, data, delt)
              if (any(is.na(fit$coef)))
                next
              results[i,leaveOutIndex] <- NA  # account for left-out country
              results[i,!is.na(results[i,])] <- fit$coef
              break
            }
            predictions[!is.na(predictions[,1,i]),,i] <- as.matrix(tmpPredict[,-1])
            colnames(predictions) <- colnames(tmpPredict)[-1]
          }

          varNames <- c(names(fit$coef)[1:(2+quadratic*2)],
                         paste(countryNames), paste('t:',countryNames,sep=''),
                         paste('t2:',countryNames,sep=''))
          means <- apply(results, 2, mean, na.rm=T)
          ses <- apply(results, 2, sd, na.rm=T)
          pvals <- 2*(1-pt(means/ses, df))
          colnames(results) <- varNames; names(means) <- varNames
          names(ses) <-varNames; names(pvals) <- varNames
          model[[cr]][[set]][[reg]][[ordr]] <- list(means = means, SEs = ses, pvals=pvals,
                                     countryNames=countryNames, pred=predictions, results=results)
        }
      }
    }
  }
  print(Sys.time()-startTime)
  model
}


### Output mean predictions and associated confidence intervals
### for a given set of models and predictions in format output
### by BootstrapModels, e.g.
MeanPredAndCI <- function(modelList, alpha) {
  means = li = ui = list()
  for (cr in names(modelList)) {
    means[[cr]] = li[[cr]] = ui[[cr]] = list()
    for (set in names(modelList[[cr]])) {
      means[[cr]][[set]] =  li[[cr]][[set]] = ui[[cr]][[set]] = list()
      for (reg in 1:nreg) {
        means[[cr]][[set]][[reg]] = li[[cr]][[set]][[reg]] = ui[[cr]][[set]][[reg]] = list()
        for (ordr in 1:2) {
          #print(paste('cr = ', cr, ', set = ', set, ', reg = ', reg, ', order = ', ordr))
          if (all(is.na(modelList[[cr]][[set]][[reg]][[ordr]][[1]]))) {
            means[[cr]][[set]][[reg]][[ordr]] <- NA
            li[[cr]][[set]][[reg]][[ordr]] <- NA
            ui[[cr]][[set]][[reg]][[ordr]] <- NA
            next
          }
          predictions <- modelList[[cr]][[set]][[reg]][[ordr]]$pred
          countryNames <- modelList[[cr]][[set]][[reg]][[ordr]]$countryNames
          mean.frame = li.frame = ui.frame = array(dim=c(length(countryNames),
                                             length(predictions[1,,1])))
          # build CI
          yieldChange <- array(dim=dim(predictions))
          for (cntry in 1:length(countryNames)) {
            for (dlt in 1:length(predictions[1,,1])) {
              yieldChange[cntry,dlt,] <- 100*predictions[cntry,dlt,]
              li.frame[cntry,dlt] <- quantile(yieldChange[cntry,dlt,], alpha/2, na.rm=T)
              ui.frame[cntry,dlt] <- quantile(yieldChange[cntry,dlt,], 1-(alpha/2), na.rm=T)
              mean.frame[cntry,dlt] <- mean(yieldChange[cntry,dlt,], na.rm=T)
            }
          }
          colnames(li.frame) = colnames(ui.frame) = colnames(mean.frame) =
            paste('plus',delt,sep='')
          means[[cr]][[set]][[reg]][[ordr]] <- data.frame(countryNames,mean.frame)
          li[[cr]][[set]][[reg]][[ordr]] <- data.frame(countryNames,li.frame)
          ui[[cr]][[set]][[reg]][[ordr]] <- data.frame(countryNames,ui.frame)
        }
      }
    }
  }
  preds <- list(means=means, li = li, ui = ui)
}

MeanPredAndSE <- function(modelList) {
  means = se = list()
  for (cr in names(modelList)) {
    means[[cr]] = se[[cr]] = list()
    for (set in names(modelList[[cr]])) {
      means[[cr]][[set]] =  se[[cr]][[set]] = list()
      for (reg in 1:nreg) {
        means[[cr]][[set]][[reg]] = se[[cr]][[set]][[reg]] = list()
        for (ordr in 1:2) {
          #print(paste('cr = ', cr, ', set = ', set, ', reg = ', reg, ', order = ', ordr))
          if (all(is.na(modelList[[cr]][[set]][[reg]][[ordr]][[1]]))) {
            means[[cr]][[set]][[reg]][[ordr]] <- NA
            se[[cr]][[set]][[reg]][[ordr]] <- NA
            next
          }
          predictions <- modelList[[cr]][[set]][[reg]][[ordr]]$pred
          countryNames <- modelList[[cr]][[set]][[reg]][[ordr]]$countryNames
          mean.frame = se.frame = array(dim=c(length(countryNames),
                                              length(predictions[1,,1])))
          # extract standard errors
          yieldChange <- array(dim=dim(predictions))
          for (cntry in 1:length(countryNames)) {
            for (dlt in 1:length(predictions[1,,1])) {
              yieldChange[cntry,dlt,] <- 100*predictions[cntry,dlt,]
              se.frame[cntry,dlt] <- sd(yieldChange[cntry,dlt,], na.rm=T)
              mean.frame[cntry,dlt] <- mean(yieldChange[cntry,dlt,], na.rm=T)
            }
          }
          colnames(se.frame) = colnames(mean.frame) =
            paste('plus',delt,sep='')
          means[[cr]][[set]][[reg]][[ordr]] <- data.frame(countryNames,mean.frame)
          se[[cr]][[set]][[reg]][[ordr]] <- data.frame(countryNames,se.frame)
        }
      }
    }
  }
  preds <- list(means=means, se = se)
}



### Create data frame by crop with pct change in yield at each degree tavg change
AggregateByCountry <- function(pred) {
  # don't want to pass f into function each time, so I am assuming f is loaded
  hold.bycrop <- list()
  for (set in names(pred[[1]])) {
    hold.meth <- list()
    for (cr in names(pred)) {
      hold.meth[[cr]] <- list()
      for (ordr in 1:2) {
        save.pred <- data.frame(orig.panel$Country[!di],orig.panel$Region[!di],
                                orig.panel$FAO_code[!di])
        colnames(save.pred) <- c('country','region','faocode')
        temp=c()
        for (reg in 1:nreg) {
          if (!all(is.na(pred[[cr]][[set]][[reg]][[ordr]]))) {    #coef estimates
            predy <- pred[[cr]][[set]][[reg]][[ordr]]
            temp <- rbind(temp,predy)
          }
        }
        save.pred <- merge(save.pred,data.frame(temp),by.x=1,by.y=1)
        hold.meth[[cr]][[ordr]] <- save.pred
      }
    }
    hold.bycrop[[set]] <- hold.meth
  }
  hold.bycrop
}


### Compute production weights for each nation and aggregate into regions
Regionalize <- function(hold) {
  holdList <- list()
  for (set in names(hold)) {
    hold.bycrop <- hold[[set]]
    hold.all <- c()
    for (crop.i in 1:length(crops)) {
      avgprod <- get.avg(crop.i,'Production',yrs=c(1998,2002),plot.flag=0)
      names(avgprod) <- tolower(names(avgprod))
      hold.imp <- c()
      for (pred.meth in c(1:2)) {
        save.pred <- hold.bycrop[[crop.i]][[pred.meth]]
        temp <- merge(avgprod,save.pred,by.x=1,by.y=3)
        pcols <- which(substr(names(temp),1,4)=='plus')
        reg.tot <- c()
        for (reg.i in 1:nreg) {
          index <- which(is.finite(temp$plus1) & is.finite(temp$production)
                         & temp$region == reg.i)
          totprod <- sum(temp$production[index])
          pwi <- apply(temp[index,pcols]*temp$production[index],2,sum)/totprod
          reg.tot <- rbind(reg.tot,c(reg.i,totprod,pwi))
        }
        use.reg <- which(is.finite(reg.tot[,3]))
        globprod <- sum(reg.tot[use.reg,2])
        if (!is.null(dim(reg.tot[use.reg,-c(1:2)]*reg.tot[use.reg,2]))) {
          globimp <- apply(reg.tot[use.reg,-c(1:2)]*reg.tot[use.reg,2],2,sum)/globprod
        } else {
          globimp <- (reg.tot[use.reg,-c(1:2)]*reg.tot[use.reg,2])/globprod
        }
        hold.imp <- cbind(hold.imp,rbind(reg.tot,c(NA,globprod,globimp)))
      }
      hold.all <- rbind(hold.all,cbind(crops[crop.i],c(reg.names,'global'),round(hold.imp,2)))
    }
    holdList[[set]] <- hold.all
  }
  holdList
}


boot.models <- BootstrapModels(panels, 500, delt)
pred.boot95 <- MeanPredAndCI(boot.models, 0.05)
pred.bootSE <- MeanPredAndSE(boot.models)

hold.bycrop <- AggregateByCountry(pred.boot95$means)
hold.bycrop.se <- AggregateByCountry(pred.bootSE$se)
hold.bycrop.li <- AggregateByCountry(pred.boot95$li)
hold.bycrop.ui <- AggregateByCountry(pred.boot95$ui)
hold.all <- Regionalize(hold.bycrop)
hold.all.li <- Regionalize(hold.bycrop.li)
hold.all.ui <- Regionalize(hold.bycrop.ui)

save(boot.models, pred.boot95, pred.bootSE, hold.bycrop, hold.bycrop.se,
     hold.bycrop.li, hold.bycrop.ui, hold.all, hold.all.li, hold.all.ui,
     file=paste(qgdir,'output\\6crop\\aggregation_fe_boot_tavgIncreaseDeg_6crop_countryxt2_500.RData',sep=''))
